#include "Body.hpp"

#define THICK		4		// Thickness of Body (y-axis)

#define UPARM_W1_1	18		// Top inner x-cord
#define UPARM_W1_2	25		// Top outer x-cord
#define UPARM_W2_1	18		// Bottom inner x-cord
#define UPARM_W2_2	25		// Bottom outer x-cord
#define UPARM_H1	50		// Top Height
#define UPARM_H2	15		// Bottom Height
#define UPARM_DIST	(UPARM_H1-UPARM_H2)

#define LOARM_W1_1	18		// Top inner x-cord
#define LOARM_W1_2	25		// Top outer x-cord
#define LOARM_W2_1	18		// Bottom inner x-cord
#define LOARM_W2_2	25		// Bottom outer x-cord
#define LOARM_H1	15		// Top Height
#define LOARM_H2	-10		// Bottom Height
#define LOARM_DIST	(LOARM_H1-LOARM_H2)

#define HAND_W1_1	18		// Top inner x-cord
#define HAND_W1_2	25		// Top outer x-cord
#define HAND_W2_1	18		// Bottom inner x-cord
#define HAND_W2_2	24		// Bottom outer x-cord
#define HAND_H1		-10		// Top Height
#define HAND_H2		-25		// Bottom Height
#define HAND_DIST	(HAND_H1-HAND_H2)
#define HAND_THICK	7

#define SHO_W1		18		// Top x-cord
#define SHO_W2		18		// Bottom x-cord
#define SHO_H1		50		// Top Height
#define SHO_H2		40		// Bottom Height

#define TOR_W1		15		// Top x-cord
#define TOR_W2		12		// Bottom x-cord
#define TOR_H1		40		// Top Height
#define TOR_H2		0		// Bottom Height

#define PEL_W1		12		// Top x-cord
#define PEL_W2		12		// Bottom x-cord
#define PEL_H1		0		// Top Height
#define PEL_H2		-10		// Bottom Height

#define THI_W1_1	2		// Top inner x-cord
#define THI_W1_2	12		// Top outer x-cord
#define THI_W2_1	2		// Bottom inner x-cord
#define THI_W2_2	12		// Bottom outer x-cord
#define THI_H1		-10		// Top Height
#define THI_H2		-50		// Bottom Height

#define SHIN_W1_1	2		// Top inner x-cord
#define SHIN_W1_2	12		// Top outer x-cord
#define SHIN_W2_1	2		// Bottom inner x-cord
#define SHIN_W2_2	12		// Bottom outer x-cord
#define SHIN_H1		-50		// Top Height
#define SHIN_H2		-90		// Bottom Height

#define FEET_W1_1	2		// Top inner x-cord
#define FEET_W1_2	12		// Top outer x-cord
#define FEET_W2_1	2		// Bottom inner x-cord
#define FEET_W2_2	12		// Bottom outer x-cord
#define FEET_H1		-90		// Top Height
#define FEET_H2		-95		// Bottom Height
#define FEET_THICK	10

Body::Body()
{
	boxImg.Create(128,128,false);
	boxImg.Fill(150,150,150);
	for (int i=0; i < 128; i++)
	{
		boxImg.SetPixel(i, 0, 0, 0, 0);
		boxImg.SetPixel(i, 127, 0, 0, 0);
		boxImg.SetPixel(0, i, 0, 0, 0);
		boxImg.SetPixel(127, i, 0, 0, 0);
	}

	redImg.Create(128,128,false);
	redImg.Fill(255,0,0);
	for (int i=0; i < 128; i++)
	{
		redImg.SetPixel(i, 0, 0, 0, 0);
		redImg.SetPixel(i, 127, 0, 0, 0);
		redImg.SetPixel(0, i, 0, 0, 0);
		redImg.SetPixel(127, i, 0, 0, 0);
	}

	blueImg.Create(128,128,false);
	blueImg.Fill(0,0,255);
	for (int i=0; i < 128; i++)
	{
		blueImg.SetPixel(i, 0, 0, 0, 0);
		blueImg.SetPixel(i, 127, 0, 0, 0);
		blueImg.SetPixel(0, i, 0, 0, 0);
		blueImg.SetPixel(127, i, 0, 0, 0);
	}

	faceImg1.Create(128,128,false);
	faceImg1.Fill(250,250,0);
	for (int i=0; i < 128; i++)
	{
		faceImg1.SetPixel(i, 0, 0, 0, 0);
		faceImg1.SetPixel(i, 127, 0, 0, 0);
		faceImg1.SetPixel(0, i, 0, 0, 0);
		faceImg1.SetPixel(127, i, 0, 0, 0);
	}

	faceImg2.Load("Images\\Smiley.bmp",false);
	faceImg3.Load("Images\\NotSmiley.bmp",false);

	bigRedImg.Load("Images\\BigRed.bmp",false);

	elbow_phi = 0;
	elbow_theta = 0;
	wrist_phi = 0;
	wrist_psi = 0;

	// Feet
	fe1_pt[0].Set(-FEET_W1_2,-THICK-FEET_THICK, FEET_H1);
	fe1_pt[1].Set(-FEET_W1_1,-THICK-FEET_THICK, FEET_H1);
	fe1_pt[2].Set(-FEET_W1_2, THICK,			FEET_H1);
	fe1_pt[3].Set(-FEET_W1_1, THICK,			FEET_H1);
	fe1_pt[4].Set(-FEET_W2_2,-THICK-FEET_THICK, FEET_H2);
	fe1_pt[5].Set(-FEET_W2_1,-THICK-FEET_THICK, FEET_H2);
	fe1_pt[6].Set(-FEET_W2_2, THICK,			FEET_H2);
	fe1_pt[7].Set(-FEET_W2_1, THICK,			FEET_H2);

	fe2_pt[0].Set( FEET_W1_2,-THICK-FEET_THICK, FEET_H1);
	fe2_pt[1].Set( FEET_W1_1,-THICK-FEET_THICK, FEET_H1);
	fe2_pt[2].Set( FEET_W1_2, THICK,			FEET_H1);
	fe2_pt[3].Set( FEET_W1_1, THICK,			FEET_H1);
	fe2_pt[4].Set( FEET_W2_2,-THICK-FEET_THICK, FEET_H2);
	fe2_pt[5].Set( FEET_W2_1,-THICK-FEET_THICK, FEET_H2);
	fe2_pt[6].Set( FEET_W2_2, THICK,			FEET_H2);
	fe2_pt[7].Set( FEET_W2_1, THICK,			FEET_H2);

	// Shin
	sh1_pt[0].Set(-SHIN_W1_2,-THICK, SHIN_H1);
	sh1_pt[1].Set(-SHIN_W1_1,-THICK, SHIN_H1);
	sh1_pt[2].Set(-SHIN_W1_2, THICK, SHIN_H1);
	sh1_pt[3].Set(-SHIN_W1_1, THICK, SHIN_H1);
	sh1_pt[4].Set(-SHIN_W2_2,-THICK, SHIN_H2);
	sh1_pt[5].Set(-SHIN_W2_1,-THICK, SHIN_H2);
	sh1_pt[6].Set(-SHIN_W2_2, THICK, SHIN_H2);
	sh1_pt[7].Set(-SHIN_W2_1, THICK, SHIN_H2);

	sh2_pt[0].Set( SHIN_W1_1,-THICK, SHIN_H1);
	sh2_pt[1].Set( SHIN_W1_2,-THICK, SHIN_H1);
	sh2_pt[2].Set( SHIN_W1_1, THICK, SHIN_H1);
	sh2_pt[3].Set( SHIN_W1_2, THICK, SHIN_H1);
	sh2_pt[4].Set( SHIN_W2_1,-THICK, SHIN_H2);
	sh2_pt[5].Set( SHIN_W2_2,-THICK, SHIN_H2);
	sh2_pt[6].Set( SHIN_W2_1, THICK, SHIN_H2);
	sh2_pt[7].Set( SHIN_W2_2, THICK, SHIN_H2);

	// Thigh
	th1_pt[0].Set(-THI_W1_2,-THICK, THI_H1);
	th1_pt[1].Set(-THI_W1_1,-THICK, THI_H1);
	th1_pt[2].Set(-THI_W1_2, THICK, THI_H1);
	th1_pt[3].Set(-THI_W1_1, THICK, THI_H1);
	th1_pt[4].Set(-THI_W2_2,-THICK, THI_H2);
	th1_pt[5].Set(-THI_W2_1,-THICK, THI_H2);
	th1_pt[6].Set(-THI_W2_2, THICK, THI_H2);
	th1_pt[7].Set(-THI_W2_1, THICK, THI_H2);

	th2_pt[0].Set( THI_W1_1,-THICK, THI_H1);
	th2_pt[1].Set( THI_W1_2,-THICK, THI_H1);
	th2_pt[2].Set( THI_W1_1, THICK, THI_H1);
	th2_pt[3].Set( THI_W1_2, THICK, THI_H1);
	th2_pt[4].Set( THI_W2_1,-THICK, THI_H2);
	th2_pt[5].Set( THI_W2_2,-THICK, THI_H2);
	th2_pt[6].Set( THI_W2_1, THICK, THI_H2);
	th2_pt[7].Set( THI_W2_2, THICK, THI_H2);

	// Pelvis
	pel_pt[0].Set(-PEL_W1,-THICK, PEL_H1);
	pel_pt[1].Set( PEL_W1,-THICK, PEL_H1);
	pel_pt[2].Set(-PEL_W1, THICK, PEL_H1);
	pel_pt[3].Set( PEL_W1, THICK, PEL_H1);
	pel_pt[4].Set(-PEL_W2,-THICK, PEL_H2);
	pel_pt[5].Set( PEL_W2,-THICK, PEL_H2);
	pel_pt[6].Set(-PEL_W2, THICK, PEL_H2);
	pel_pt[7].Set( PEL_W2, THICK, PEL_H2);

	// Torso
	tor_pt[0].Set(-TOR_W1,-THICK, TOR_H1);
	tor_pt[1].Set( TOR_W1,-THICK, TOR_H1);
	tor_pt[2].Set(-TOR_W1, THICK, TOR_H1);
	tor_pt[3].Set( TOR_W1, THICK, TOR_H1);
	tor_pt[4].Set(-TOR_W2,-THICK, TOR_H2);
	tor_pt[5].Set( TOR_W2,-THICK, TOR_H2);
	tor_pt[6].Set(-TOR_W2, THICK, TOR_H2);
	tor_pt[7].Set( TOR_W2, THICK, TOR_H2);

	// Shoulder
	sho_pt[0].Set(-SHO_W1,-THICK, SHO_H1);
	sho_pt[1].Set( SHO_W1,-THICK, SHO_H1);
	sho_pt[2].Set(-SHO_W1, THICK, SHO_H1);
	sho_pt[3].Set( SHO_W1, THICK, SHO_H1);
	sho_pt[4].Set(-SHO_W2,-THICK, SHO_H2);
	sho_pt[5].Set( SHO_W2,-THICK, SHO_H2);
	sho_pt[6].Set(-SHO_W2, THICK, SHO_H2);
	sho_pt[7].Set( SHO_W2, THICK, SHO_H2);

	// Neck
	nec_pt[0].Set(-2,-2, 55);
	nec_pt[1].Set( 2,-2, 55);
	nec_pt[2].Set(-2, 2, 55);
	nec_pt[3].Set( 2, 2, 55);
	nec_pt[4].Set(-2,-2, 50);
	nec_pt[5].Set( 2,-2, 50);
	nec_pt[6].Set(-2, 2, 50);
	nec_pt[7].Set( 2, 2, 50);

	// Head
	hed_pt[0].Set(-10,-THICK, 80);
	hed_pt[1].Set( 10,-THICK, 80);
	hed_pt[2].Set(-10, THICK, 80);
	hed_pt[3].Set( 10, THICK, 80);
	hed_pt[4].Set(-7,-THICK, 55);
	hed_pt[5].Set( 7,-THICK, 55);
	hed_pt[6].Set(-7, THICK, 55);
	hed_pt[7].Set( 7, THICK, 55);

	// Arms
	upArm1_pt[0].Set(-UPARM_W1_2,-THICK, UPARM_H1);
	upArm1_pt[1].Set(-UPARM_W1_1,-THICK, UPARM_H1);
	upArm1_pt[2].Set(-UPARM_W1_2, THICK, UPARM_H1);
	upArm1_pt[3].Set(-UPARM_W1_1, THICK, UPARM_H1);
	upArm1_pt[4].Set(-UPARM_W2_2,-THICK, UPARM_H2);
	upArm1_pt[5].Set(-UPARM_W2_1,-THICK, UPARM_H2);
	upArm1_pt[6].Set(-UPARM_W2_2, THICK, UPARM_H2);
	upArm1_pt[7].Set(-UPARM_W2_1, THICK, UPARM_H2);

	upArm2_pt[0].Set( UPARM_W1_2,-THICK, UPARM_H1);
	upArm2_pt[1].Set( UPARM_W1_1,-THICK, UPARM_H1);
	upArm2_pt[2].Set( UPARM_W1_2, THICK, UPARM_H1);
	upArm2_pt[3].Set( UPARM_W1_1, THICK, UPARM_H1);
	upArm2_pt[4].Set( UPARM_W2_2,-THICK, UPARM_H2);
	upArm2_pt[5].Set( UPARM_W2_1,-THICK, UPARM_H2);
	upArm2_pt[6].Set( UPARM_W2_2, THICK, UPARM_H2);
	upArm2_pt[7].Set( UPARM_W2_1, THICK, UPARM_H2);

	loArm1_pt[0].Set(-LOARM_W1_2,-THICK, LOARM_H1);
	loArm1_pt[1].Set(-LOARM_W1_1,-THICK, LOARM_H1);
	loArm1_pt[2].Set(-LOARM_W1_2, THICK, LOARM_H1);
	loArm1_pt[3].Set(-LOARM_W1_1, THICK, LOARM_H1);
	loArm1_pt[4].Set(-LOARM_W2_2,-THICK, LOARM_H2);
	loArm1_pt[5].Set(-LOARM_W2_1,-THICK, LOARM_H2);
	loArm1_pt[6].Set(-LOARM_W2_2, THICK, LOARM_H2);
	loArm1_pt[7].Set(-LOARM_W2_1, THICK, LOARM_H2);

	loArm2_pt[0].Set( LOARM_W1_2,-THICK, LOARM_H1);
	loArm2_pt[1].Set( LOARM_W1_1,-THICK, LOARM_H1);
	loArm2_pt[2].Set( LOARM_W1_2, THICK, LOARM_H1);
	loArm2_pt[3].Set( LOARM_W1_1, THICK, LOARM_H1);
	loArm2_pt[4].Set( LOARM_W2_2,-THICK, LOARM_H2);
	loArm2_pt[5].Set( LOARM_W2_1,-THICK, LOARM_H2);
	loArm2_pt[6].Set( LOARM_W2_2, THICK, LOARM_H2);
	loArm2_pt[7].Set( LOARM_W2_1, THICK, LOARM_H2);

	hand1_pt[0].Set(-HAND_W1_2,-HAND_THICK, HAND_H1);
	hand1_pt[1].Set(-HAND_W1_1,-HAND_THICK, HAND_H1);
	hand1_pt[2].Set(-HAND_W1_2, HAND_THICK, HAND_H1);
	hand1_pt[3].Set(-HAND_W1_1, HAND_THICK, HAND_H1);
	hand1_pt[4].Set(-HAND_W2_2,-HAND_THICK, HAND_H2);
	hand1_pt[5].Set(-HAND_W2_1,-HAND_THICK, HAND_H2);
	hand1_pt[6].Set(-HAND_W2_2, HAND_THICK, HAND_H2);
	hand1_pt[7].Set(-HAND_W2_1, HAND_THICK, HAND_H2);

	hand2_pt[0].Set( HAND_W1_2,-HAND_THICK, HAND_H1);
	hand2_pt[1].Set( HAND_W1_1,-HAND_THICK, HAND_H1);
	hand2_pt[2].Set( HAND_W1_2, HAND_THICK, HAND_H1);
	hand2_pt[3].Set( HAND_W1_1, HAND_THICK, HAND_H1);
	hand2_pt[4].Set( HAND_W2_2,-HAND_THICK, HAND_H2);
	hand2_pt[5].Set( HAND_W2_1,-HAND_THICK, HAND_H2);
	hand2_pt[6].Set( HAND_W2_2, HAND_THICK, HAND_H2);
	hand2_pt[7].Set( HAND_W2_1, HAND_THICK, HAND_H2);

	for (int i = 0; i < 8; i++)
	{
		orig_fe1_pt[i] = fe1_pt[i];
		orig_fe2_pt[i] = fe2_pt[i];
		orig_sh1_pt[i] = sh1_pt[i];
		orig_sh2_pt[i] = sh2_pt[i];
		orig_th1_pt[i] = th1_pt[i];
		orig_th2_pt[i] = th2_pt[i];
		orig_pel_pt[i] = pel_pt[i];
		orig_tor_pt[i] = tor_pt[i];
		orig_sho_pt[i] = sho_pt[i];
		orig_nec_pt[i] = nec_pt[i];
		orig_hed_pt[i] = hed_pt[i];
		orig_upArm1_pt[i] = upArm1_pt[i];
		orig_upArm2_pt[i] = upArm2_pt[i];
		orig_loArm1_pt[i] = loArm1_pt[i];
		orig_loArm2_pt[i] = loArm2_pt[i];
		orig_hand1_pt[i] = hand1_pt[i];
		orig_hand2_pt[i] = hand2_pt[i];
	}

	float avgX, avgY, avgZ;
	avgX = (orig_loArm1_pt[0].x + orig_loArm1_pt[1].x + orig_loArm1_pt[2].x + orig_loArm1_pt[3].x) / 4;
	avgY = (orig_loArm1_pt[0].y + orig_loArm1_pt[1].y + orig_loArm1_pt[2].y + orig_loArm1_pt[3].y) / 4;
	avgZ = (orig_loArm1_pt[0].z + orig_loArm1_pt[1].z + orig_loArm1_pt[2].z + orig_loArm1_pt[3].z) / 4;
	orig_loArm1_avg.Set(avgX, avgY, avgZ);

	avgX = (orig_hand1_pt[0].x + orig_hand1_pt[1].x + orig_hand1_pt[2].x + orig_hand1_pt[3].x) / 4;
	avgY = (orig_hand1_pt[0].y + orig_hand1_pt[1].y + orig_hand1_pt[2].y + orig_hand1_pt[3].y) / 4;
	avgZ = (orig_hand1_pt[0].z + orig_hand1_pt[1].z + orig_hand1_pt[2].z + orig_hand1_pt[3].z) / 4;
	orig_hand1_avg.Set(avgX, avgY, avgZ);

	Reset();

}

void Body::Draw()
{
	// Feet
	DrawBoxPts(&boxImg, fe1_pt);
	DrawBoxPts(&boxImg, fe2_pt);
	
	// Shins
	DrawBoxPts(&boxImg, sh1_pt);
	DrawBoxPts(&boxImg, sh2_pt);

	// Thighs
	DrawBoxPts(&blueImg, th1_pt);
	DrawBoxPts(&blueImg, th2_pt);

	// Pelvis
	DrawBoxPts(&blueImg, pel_pt);

	// Torso
	//DrawBoxPts(&boxImg, tor_pt);
	DrawShirt(tor_pt);

	// Shoulder
	DrawBoxPts(&redImg, sho_pt);

	// Neck
	DrawBoxPts(&boxImg, nec_pt);

	// Arms
	DrawBoxPts(&redImg, upArm1_pt);
	DrawBoxPts(&redImg, upArm2_pt);
	DrawBoxPts(&redImg, loArm1_pt);
	DrawBoxPts(&redImg, loArm2_pt);

	// Hands
	DrawBoxPts(&boxImg, hand1_pt);
	DrawBoxPts(&boxImg, hand2_pt);

	// Head
	DrawFace(hed_pt);

	//GameX.DrawPoint3D(ColorX(255,255,255), tempPt);
	//GameX.DrawLine3D(ColorX(255,255,255), Vector3DF(50,0,50), Vector3DF(50,0,50)+ tempPt);
}

void Body::DrawBoxPts(Vector3DF pt1, Vector3DF pt2, Vector3DF pt3, Vector3DF pt4, Vector3DF pt5, Vector3DF pt6, Vector3DF pt7, Vector3DF pt8)
{
	GameX.DrawTexturedPolygon3D( &boxImg, pt3, pt4, pt1, pt2 );
	GameX.DrawTexturedPolygon3D( &boxImg, pt5, pt6, pt7, pt8 );
	GameX.DrawTexturedPolygon3D( &boxImg, pt1, pt2, pt5, pt6 );
	GameX.DrawTexturedPolygon3D( &boxImg, pt2, pt4, pt6, pt8 );
	GameX.DrawTexturedPolygon3D( &boxImg, pt4, pt3, pt8, pt7 );
	GameX.DrawTexturedPolygon3D( &boxImg, pt3, pt1, pt7, pt5 );
}

void Body::DrawBoxPts(ImageX *img, Vector3DF pt[])
{
	GameX.DrawTexturedPolygon3D( img, pt[2], pt[3], pt[0], pt[1] );
	GameX.DrawTexturedPolygon3D( img, pt[4], pt[5], pt[6], pt[7] );
	GameX.DrawTexturedPolygon3D( img, pt[0], pt[1], pt[4], pt[5] );
	GameX.DrawTexturedPolygon3D( img, pt[1], pt[3], pt[5], pt[7] );
	GameX.DrawTexturedPolygon3D( img, pt[3], pt[2], pt[7], pt[6] );
	GameX.DrawTexturedPolygon3D( img, pt[2], pt[0], pt[6], pt[4] );
}

void Body::DrawFace( Vector3DF pt[] )
{
	GameX.DrawTexturedPolygon3D( &faceImg1, pt[2], pt[3], pt[0], pt[1] );
	GameX.DrawTexturedPolygon3D( &faceImg1, pt[4], pt[5], pt[6], pt[7] );
	if (wrist_phi > elbow_phi)
		GameX.DrawTexturedPolygon3D( &faceImg3, pt[0], pt[1], pt[4], pt[5] );
	else
		GameX.DrawTexturedPolygon3D( &faceImg2, pt[0], pt[1], pt[4], pt[5] );
	GameX.DrawTexturedPolygon3D( &faceImg1, pt[1], pt[3], pt[5], pt[7] );
	GameX.DrawTexturedPolygon3D( &faceImg1, pt[3], pt[2], pt[7], pt[6] );
	GameX.DrawTexturedPolygon3D( &faceImg1, pt[2], pt[0], pt[6], pt[4] );
}

void Body::DrawShirt( Vector3DF pt[] )
{
	GameX.DrawTexturedPolygon3D( &redImg, pt[2], pt[3], pt[0], pt[1] );
	GameX.DrawTexturedPolygon3D( &redImg, pt[4], pt[5], pt[6], pt[7] );
	GameX.DrawTexturedPolygon3D( &bigRedImg, pt[0], pt[1], pt[4], pt[5] );
	GameX.DrawTexturedPolygon3D( &redImg, pt[1], pt[3], pt[5], pt[7] );
	GameX.DrawTexturedPolygon3D( &redImg, pt[3], pt[2], pt[7], pt[6] );
	GameX.DrawTexturedPolygon3D( &redImg, pt[2], pt[0], pt[6], pt[4] );
}

void Body::RotateAroundPoint(Vector3DF& position, Vector3DF vCenter, float angle, Vector3DF vAxis)
{
	Vector3DF vNewPosition;	

	// Get the Vector from our position to the center we are rotating around
	Vector3DF vPos = position - vCenter;

	// Calculate the sine and cosine of the angle once
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// Find the new x position for the new rotated point
	vNewPosition.x  = (cosTheta + (1 - cosTheta) * vAxis.x * vAxis.x)		* vPos.x;
	vNewPosition.x += ((1 - cosTheta) * vAxis.x * vAxis.y - vAxis.z * sinTheta)	* vPos.y;
	vNewPosition.x += ((1 - cosTheta) * vAxis.x * vAxis.z + vAxis.y * sinTheta)	* vPos.z;

	// Find the new y position for the new rotated point
	vNewPosition.y  = ((1 - cosTheta) * vAxis.x * vAxis.y + vAxis.z * sinTheta)	* vPos.x;
	vNewPosition.y += (cosTheta + (1 - cosTheta) * vAxis.y * vAxis.y)		* vPos.y;
	vNewPosition.y += ((1 - cosTheta) * vAxis.y * vAxis.z - vAxis.x * sinTheta)	* vPos.z;

	// Find the new z position for the new rotated point
	vNewPosition.z  = ((1 - cosTheta) * vAxis.x * vAxis.z - vAxis.y * sinTheta)	* vPos.x;
	vNewPosition.z += ((1 - cosTheta) * vAxis.y * vAxis.z + vAxis.x * sinTheta)	* vPos.y;
	vNewPosition.z += (cosTheta + (1 - cosTheta) * vAxis.z * vAxis.z)		* vPos.z;

	position = vCenter + vNewPosition;
}

void Body::RotateAroundAxisX(Vector3DF &newPosition, Vector3DF vPoint, Vector3DF vCenter, float angle)
{
	Vector3DF vNewPosition;	

	// Get the Vector from our position to the center we are rotating around
	Vector3DF vPos = vPoint - vCenter;

	// Calculate the sine and cosine of the angle once
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// Find the new x position for the new rotated point
	vNewPosition.x  = vPos.x;

	// Find the new y position for the new rotated point
	vNewPosition.y  = (cosTheta * vPos.y) - (sinTheta * vPos.z);

	// Find the new z position for the new rotated point
	vNewPosition.z  = (sinTheta * vPos.y) + (cosTheta * vPos.z);

	newPosition = (vCenter + vNewPosition);
}

void Body::RotateAroundAxisY(Vector3DF &newPosition, Vector3DF vPoint, Vector3DF vCenter, float angle)
{
	Vector3DF vNewPosition;	

	// Get the Vector from our position to the center we are rotating around
	Vector3DF vPos = vPoint - vCenter;

	// Calculate the sine and cosine of the angle once
	float cosTheta = (float)cos(angle);
	float sinTheta = (float)sin(angle);

	// Find the new x position for the new rotated point
	vNewPosition.x  = (cosTheta * vPos.x) + (sinTheta * vPos.z);

	// Find the new y position for the new rotated point
	vNewPosition.y  = vPos.y;

	// Find the new z position for the new rotated point
	vNewPosition.z  = (cosTheta * vPos.z) - (sinTheta * vPos.x);

	newPosition = vCenter + vNewPosition;
}



void Body::RotateElbow(float phi, float theta)
{
	theta *= 1/60.0f;
	phi *= -1.0f/60.0f;
	phi -= PI_2;

	Vector3DF rotatePoint( (-UPARM_W1_1-UPARM_W1_2)/2, 0, UPARM_H1-THICK );

	for (int i = 0; i < 8; i++)
	{
		RotateAroundAxisY( upArm1_pt[i],	orig_upArm1_pt[i],		rotatePoint, theta);
		RotateAroundAxisY( loArm1_pt[i],	orig_loArm1_pt[i],		rotatePoint, theta);
		RotateAroundAxisY( hand1_pt[i],		orig_hand1_pt[i],		rotatePoint, theta);
	}
	
	for (int i = 0; i < 8; i++)
	{
		RotateAroundAxisX( upArm1_pt[i],	upArm1_pt[i],	rotatePoint, phi );
		RotateAroundAxisX( loArm1_pt[i],	loArm1_pt[i],	rotatePoint, phi );
		RotateAroundAxisX( hand1_pt[i],		hand1_pt[i],	rotatePoint, phi );
	}

	elbow_phi = phi;
	elbow_theta = theta;
}

void Body::RotateWrist(float phi, float psi)
{
	phi *= -1.0f;
	phi -= PI_2;

	float avgX, avgY, avgZ;
	avgX = (loArm1_pt[0].x + loArm1_pt[1].x + loArm1_pt[2].x + loArm1_pt[3].x) / 4;
	avgY = (loArm1_pt[0].y + loArm1_pt[1].y + loArm1_pt[2].y + loArm1_pt[3].y) / 4;
	avgZ = (loArm1_pt[0].z + loArm1_pt[1].z + loArm1_pt[2].z + loArm1_pt[3].z) / 4;
	Vector3DF loArmRotatePoint(avgX, avgY, avgZ);
	Vector3DF handRotatePoint(avgX, avgY, avgZ-(LOARM_H1-LOARM_H2));
	tempPt = handRotatePoint;

	Vector3DF rotatePhiAxis(cosf(elbow_theta), sinf(elbow_theta), 0);
	Vector3DF rotatePsiAxis = (upArm1_pt[4] - upArm1_pt[0]).Normalize();

	for (int i = 0; i < 8; i++)
	{
		loArm1_pt[i]	=		(loArmRotatePoint-orig_loArm1_avg) + orig_loArm1_pt[i];
		hand1_pt[i]		=		(handRotatePoint-orig_hand1_avg) + orig_hand1_pt[i];

		RotateAroundPoint(loArm1_pt[i], loArmRotatePoint, phi, rotatePhiAxis);
		RotateAroundPoint(hand1_pt[i], loArmRotatePoint, phi, rotatePhiAxis);		
	}

	for (int i = 0; i < 8; i++)
	{
		RotateAroundPoint(loArm1_pt[i], loArmRotatePoint, psi, rotatePsiAxis);
		RotateAroundPoint(hand1_pt[i], loArmRotatePoint, psi, rotatePsiAxis);
	}


	wrist_phi = phi;
	wrist_psi = psi;
}

void Body::Reset()
{
	RotateElbow(0,0);
	RotateWrist(0,0);
}
